@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -29,25 +29,27 @@
     $Id$
 */
 
-/// \file content_manager.cc
-#define LOG_FAC log_facility_t::content
+/// @file content/content_manager.cc
+#define GRB_LOG_FAC GrbLogFacility::content
 
 #include "content_manager.h" // API
 
-#include <fmt/chrono.h>
-#include <regex>
-
 #include "autoscan_list.h"
 #include "cds/cds_container.h"
 #include "cds/cds_item.h"
+#include "cm_task.h"
 #include "config/config_option_enum.h"
 #include "config/config_options.h"
+#include "config/config_val.h"
 #include "config/result/autoscan.h"
+#include "context.h"
 #include "database/database.h"
+#include "exceptions.h"
 #include "import_service.h"
 #include "metadata/metadata_service.h"
 #include "update_manager.h"
 #include "upnp/clients.h"
+#include "util/generic_task.h"
 #include "util/mime.h"
 #include "util/string_converter.h"
 #include "util/timer.h"
@@ -55,19 +57,18 @@
 #include "web/session_manager.h"
 
 #ifdef HAVE_JS
-#include "layout/js_layout.h"
-#include "scripting/import_script.h"
 #include "scripting/metafile_parser_script.h"
 #include "scripting/playlist_parser_script.h"
 #include "scripting/scripting_runtime.h"
 #endif
 
-#ifdef HAVE_LASTFMLIB
+#ifdef HAVE_LASTFM
 #include "onlineservice/lastfm_scrobbler.h"
+#ifdef HAVE_LASTFMLIB
+#include <lastfmlib/lastfmscrobbler.h>
+#else
+#include "onlineservice/lastfmlib.h"
 #endif
-
-#ifdef ATRAILERS
-#include "onlineservice/atrailers_service.h"
 #endif
 
 #ifdef ONLINE_SERVICES
@@ -76,21 +77,25 @@
 #endif
 
 #ifdef HAVE_INOTIFY
-#include "autoscan_inotify.h"
+#include "content/inotify/autoscan_inotify.h"
+#include "content/inotify/scripting_inotify.h"
 #endif
 
+#include <algorithm>
+
 ContentManager::ContentManager(const std::shared_ptr<Context>& context,
     const std::shared_ptr<Server>& server, std::shared_ptr<Timer> timer)
     : config(context->getConfig())
     , mime(context->getMime())
     , database(context->getDatabase())
     , session_manager(context->getSessionManager())
+    , converterManager(context->getConverterManager())
     , context(context)
     , timer(std::move(timer))
 #ifdef HAVE_JS
     , scriptingRuntime(std::make_shared<ScriptingRuntime>())
 #endif
-#ifdef HAVE_LASTFMLIB
+#ifdef HAVE_LASTFM
     , last_fm(std::make_shared<LastFm>(context))
 #endif
 {
@@ -98,10 +103,25 @@ ContentManager::ContentManager(const std::shared_ptr<Context>& context,
 #ifdef ONLINE_SERVICES
     task_processor = std::make_shared<TaskProcessor>(config);
 #endif
-    importService = std::make_shared<ImportService>(this->context);
-    importMode = EnumOption<ImportMode>::getEnumOption(config, CFG_IMPORT_LAYOUT_MODE);
+    importService = std::make_shared<ImportService>(this->context, converterManager);
+    importMode = EnumOption<ImportMode>::getEnumOption(config, ConfigVal::IMPORT_LAYOUT_MODE);
+#ifdef HAVE_INOTIFY
+    useAsInotify = config->getBoolOption(ConfigVal::IMPORT_AUTOSCAN_USE_INOTIFY);
+#endif
+#ifdef HAVE_JS
+    scriptScanMode = EnumOption<AutoscanScanMode>::getEnumOption(config, ConfigVal::IMPORT_SCRIPTING_SCAN_MODE);
+#endif
 }
 
+#ifdef HAVE_LASTFM
+#ifndef HAVE_LASTFMLIB
+void ContentManager::initLastFM()
+{
+    last_fm->setup();
+}
+#endif
+#endif
+
 void ContentManager::run()
 {
     auto self = shared_from_this();
@@ -111,7 +131,7 @@ void ContentManager::run()
 #ifdef ONLINE_SERVICES
     task_processor->run();
 #endif
-#ifdef HAVE_LASTFMLIB
+#ifdef HAVE_LASTFM
     last_fm->run();
 #endif
     threadRunner = std::make_unique<ThreadRunner<std::condition_variable_any, std::recursive_mutex>>(
@@ -129,101 +149,105 @@ void ContentManager::run()
         throw_std_runtime_error("Could not start ContentTaskThread thread");
     }
 
+    log_debug("updateAutoscanList {} ui", AutoscanScanMode::Timed);
     autoscanList = database->getAutoscanList(AutoscanScanMode::Timed);
-    for (auto& dir : config->getAutoscanListOption(CFG_IMPORT_AUTOSCAN_TIMED_LIST)) {
-        fs::path path = dir->getLocation();
-        if (fs::is_directory(path)) {
-            dir->setObjectID(ensurePathExistence(path));
-        }
-        try {
-            autoscanList->add(dir);
-        } catch (const std::runtime_error& e) {
-            // Work around existing config sourced autoscans that were stored to the DB for reasons
-            log_warning(e.what());
-            continue;
-        }
-    }
+    auto dbScans = std::vector<AutoscanScanMode> { AutoscanScanMode::Manual };
+    auto cfScans = std::vector<ConfigVal> { ConfigVal::IMPORT_AUTOSCAN_TIMED_LIST, ConfigVal::IMPORT_AUTOSCAN_MANUAL_LIST };
 
 #ifdef HAVE_INOTIFY
-    inotify = std::make_unique<AutoscanInotify>(self);
-
-    if (config->getBoolOption(CFG_IMPORT_AUTOSCAN_USE_INOTIFY)) {
-        auto db = database->getAutoscanList(AutoscanScanMode::INotify);
-        for (std::size_t i = 0; i < db->size(); i++) {
-            auto dir = db->get(i);
-            auto path = dir->getLocation();
-            if (fs::is_directory(path)) {
-                dir->setObjectID(ensurePathExistence(path));
-            }
-            try {
-                autoscanList->add(dir);
-            } catch (const std::runtime_error& e) {
-                // Work around existing config sourced autoscans that were stored to the DB for reasons
-                log_warning(e.what());
-                continue;
+    as_inotify = std::make_unique<AutoscanInotify>(self);
+#ifdef HAVE_JS
+    if (scriptScanMode == AutoscanScanMode::INotify)
+        script_inotify = std::make_unique<ScriptingInotify>(config, scriptingRuntime);
+#endif
+    if (useAsInotify) {
+        dbScans.push_back(AutoscanScanMode::INotify);
+        cfScans.push_back(ConfigVal::IMPORT_AUTOSCAN_INOTIFY_LIST);
+    }
+#endif
+    for (auto dbMode : dbScans) {
+        log_debug("updateAutoscanList {} ui", dbMode);
+        try {
+            auto db = database->getAutoscanList(dbMode);
+            for (std::size_t i = 0; i < db->size(); i++) {
+                auto dir = db->get(i);
+                auto path = dir->getLocation();
+                if (fs::is_directory(path)) {
+                    dir->setObjectID(ensurePathExistence(path));
+                }
+                try {
+                    dir->setInvalid();
+                    autoscanList->add(dir);
+                } catch (const std::logic_error& e) {
+                    log_warning(e.what());
+                } catch (const std::runtime_error& e) {
+                    // Work around existing config sourced autoscans that were stored to the DB for reasons
+                    log_warning(e.what());
+                }
             }
+        } catch (const std::logic_error& e) {
+            log_warning(e.what());
+        } catch (const std::runtime_error& e) {
+            log_warning(e.what());
         }
-        for (const auto& dir : config->getAutoscanListOption(CFG_IMPORT_AUTOSCAN_INOTIFY_LIST)) {
+    }
+    for (auto cfMode : cfScans) {
+        log_debug("updateAutoscanList {} config", cfMode);
+        for (auto& dir : config->getAutoscanListOption(cfMode)) {
             fs::path path = dir->getLocation();
             if (fs::is_directory(path)) {
                 dir->setObjectID(ensurePathExistence(path));
             }
             try {
                 autoscanList->add(dir);
+            } catch (const std::logic_error& e) {
+                log_warning(e.what());
             } catch (const std::runtime_error& e) {
-                // Work around existing config sourced autoscans that were stored to the DB for reasons
                 log_warning(e.what());
-                continue;
             }
         }
     }
-
+#ifdef HAVE_INOTIFY
     // Start INotify thread
-    inotify->run();
+    as_inotify->run();
 #endif
 
-    auto layoutType = EnumOption<LayoutType>::getEnumOption(config, CFG_IMPORT_SCRIPTING_VIRTUAL_LAYOUT_TYPE);
+    auto layoutType = EnumOption<LayoutType>::getEnumOption(config, ConfigVal::IMPORT_SCRIPTING_VIRTUAL_LAYOUT_TYPE);
     log_debug("virtual layout Type {}", layoutType);
     layoutEnabled = (layoutType == LayoutType::Builtin || layoutType == LayoutType::Js);
 
 #ifdef ONLINE_SERVICES
     online_services = std::make_unique<OnlineServiceList>();
-
-#ifdef ATRAILERS
-    if (config->getBoolOption(CFG_ONLINE_CONTENT_ATRAILERS_ENABLED)) {
-        try {
-            auto at = std::make_shared<ATrailersService>(self);
-
-            auto i = std::chrono::seconds(config->getIntOption(CFG_ONLINE_CONTENT_ATRAILERS_REFRESH));
-            at->setRefreshInterval(i);
-
-            i = std::chrono::seconds(config->getIntOption(CFG_ONLINE_CONTENT_ATRAILERS_PURGE_AFTER));
-            at->setItemPurgeInterval(i);
-            if (config->getBoolOption(CFG_ONLINE_CONTENT_ATRAILERS_UPDATE_AT_START))
-                i = CFG_DEFAULT_UPDATE_AT_START;
-
-            auto atParam = std::make_shared<Timer::Parameter>(Timer::Parameter::IDOnlineContent, to_underlying(OnlineServiceType::OS_ATrailers));
-            at->setTimerParameter(std::move(atParam));
-            online_services->registerService(at);
-            if (i > std::chrono::seconds::zero()) {
-                timer->addTimerSubscriber(this, i, at->getTimerParameter(), true);
-            }
-        } catch (const std::runtime_error& ex) {
-            log_error("Could not setup Apple Trailers: {}", ex.what());
-        }
-    }
-#endif // ATRAILERS
-
 #endif // ONLINE_SERVICES
 
     initLayout();
 
     log_debug("autoscan_timed");
     autoscanList->notifyAll(this);
+    auto self_content = std::dynamic_pointer_cast<Content>(self);
 #ifdef HAVE_INOTIFY
-    autoscanList->initTimer(self, timer, config->getBoolOption(CFG_IMPORT_AUTOSCAN_USE_INOTIFY), inotify);
+    autoscanList->initTimer(self_content, timer, useAsInotify, as_inotify);
+#ifdef HAVE_JS
+    if (scriptScanMode == AutoscanScanMode::INotify) {
+        log_info("Adding inotify script scan");
+        script_inotify->run();
+        std::string customFolderPath = config->getOption(ConfigVal::IMPORT_SCRIPTING_CUSTOM_FOLDER);
+        std::string commonFolderPath = config->getOption(ConfigVal::IMPORT_SCRIPTING_COMMON_FOLDER);
+        if (!customFolderPath.empty())
+            script_inotify->monitor(customFolderPath);
+        if (!commonFolderPath.empty())
+            script_inotify->monitor(commonFolderPath);
+    }
+#endif
 #else
-    autoscanList->initTimer(self, timer);
+    autoscanList->initTimer(self_content, timer);
+#endif
+#ifdef HAVE_JS
+    if (scriptScanMode == AutoscanScanMode::Timed) {
+        auto param = std::make_shared<Timer::Parameter>(Timer::TimerParamType::IDScript, 0);
+        log_info("Adding timed script scan with interval {} minutes", config->getLongOption(ConfigVal::IMPORT_SCRIPTING_SCAN_INTERVAL));
+        timer->addTimerSubscriber(this, std::chrono::minutes(config->getLongOption(ConfigVal::IMPORT_SCRIPTING_SCAN_INTERVAL)), std::move(param), false);
+    }
 #endif
 }
 
@@ -254,8 +278,8 @@ void ContentManager::timerNotify(const std::shared_ptr<Timer::Parameter>& parame
     if (!parameter)
         return;
 
-    if (parameter->whoami() == Timer::Parameter::IDAutoscan) {
-        auto adir = autoscanList->get(parameter->getID());
+    if (parameter->whoami() == Timer::TimerParamType::IDAutoscan) {
+        auto adir = autoscanList ? autoscanList->get(parameter->getID()) : nullptr;
 
         // do not rescan while other scans are still active
         if (!adir || adir->getActiveScanCount() > 0 || adir->getTaskCount() > 0)
@@ -263,11 +287,16 @@ void ContentManager::timerNotify(const std::shared_ptr<Timer::Parameter>& parame
 
         rescanDirectory(adir, adir->getObjectID());
     }
+#ifdef HAVE_JS
+    else if (parameter->whoami() == Timer::TimerParamType::IDScript) {
+        scriptingRuntime->reloadFolders();
+    }
+#endif
 #ifdef ONLINE_SERVICES
-    else if (parameter->whoami() == Timer::Parameter::IDOnlineContent) {
+    else if (parameter->whoami() == Timer::TimerParamType::IDOnlineContent) {
         fetchOnlineContent(static_cast<OnlineServiceType>(parameter->getID()));
     }
-#endif // ONLINE_SERVICES
+#endif
 }
 
 void ContentManager::shutdown()
@@ -294,9 +323,13 @@ void ContentManager::shutdown()
         }
         autoscanList->updateLMinDB(*database);
 
-        autoscanList = nullptr;
+        autoscanList.reset();
 #ifdef HAVE_INOTIFY
-        inotify = nullptr;
+        as_inotify.reset();
+#ifdef HAVE_JS
+        if (scriptScanMode == AutoscanScanMode::INotify)
+            script_inotify.reset();
+#endif
 #endif
     }
 
@@ -314,19 +347,19 @@ void ContentManager::shutdown()
 
     threadRunner->join();
 
-#ifdef HAVE_LASTFMLIB
+#ifdef HAVE_LASTFM
     last_fm->shutdown();
-    last_fm = nullptr;
+    last_fm.reset();
 #endif
 #ifdef HAVE_JS
-    scriptingRuntime = nullptr;
+    scriptingRuntime.reset();
 #endif
 #ifdef ONLINE_SERVICES
     task_processor->shutdown();
-    task_processor = nullptr;
+    task_processor.reset();
 #endif
     update_manager->shutdown();
-    update_manager = nullptr;
+    update_manager.reset();
 
     log_debug("end");
 }
@@ -382,7 +415,7 @@ void ContentManager::addVirtualItem(const std::shared_ptr<CdsObject>& obj, bool
     if (ec || !isRegularFile(dirEnt, ec))
         throw_std_runtime_error("Not a file: {} - {}", path.c_str(), ec.message());
 
-    auto pcdir = database->findObjectByPath(path);
+    auto pcdir = database->findObjectByPath(path, UNUSED_CLIENT_GROUP);
     if (!pcdir) {
         auto adir = findAutoscanDirectory(path);
         pcdir = createObjectFromFile(adir, dirEnt, true, allowFifo);
@@ -409,7 +442,7 @@ std::shared_ptr<CdsObject> ContentManager::createSingleItem(
     const std::shared_ptr<AutoscanDirectory>& adir,
     const std::shared_ptr<CMAddFileTask>& task)
 {
-    auto obj = checkDatabase ? database->findObjectByPath(dirEnt.path()) : nullptr;
+    auto obj = checkDatabase ? database->findObjectByPath(dirEnt.path(), DEFAULT_CLIENT_GROUP) : nullptr;
     bool isNew = false;
 
     if (!obj) {
@@ -427,6 +460,7 @@ std::shared_ptr<CdsObject> ContentManager::createSingleItem(
     } else if (obj->isItem() && processExisting) {
         auto item = std::static_pointer_cast<CdsItem>(obj);
         getImportService(adir)->getMetadataService()->extractMetaData(item, dirEnt);
+        getImportService(adir)->getMetadataService()->attachResourceFiles(item, dirEnt);
         getImportService(adir)->updateItemData(item, item->getMimeType());
     }
     if (processExisting || isNew) {
@@ -440,7 +474,11 @@ void ContentManager::parseMetafile(const std::shared_ptr<CdsObject>& obj, const
     importService->parseMetafile(obj, path);
 }
 
-std::shared_ptr<CdsObject> ContentManager::_addFile(const fs::directory_entry& dirEnt, fs::path rootPath, AutoScanSetting& asSetting, const std::shared_ptr<CMAddFileTask>& task)
+std::shared_ptr<CdsObject> ContentManager::_addFile(
+    const fs::directory_entry& dirEnt,
+    fs::path rootPath,
+    AutoScanSetting& asSetting,
+    const std::shared_ptr<CMAddFileTask>& task)
 {
     if (!asSetting.hidden && dirEnt.path().is_relative()) {
         return nullptr;
@@ -456,13 +494,14 @@ std::shared_ptr<CdsObject> ContentManager::_addFile(const fs::directory_entry& d
 
     if (importMode == ImportMode::Gerbera) {
         std::unordered_set<int> currentContent;
-        getImportService(asSetting.adir)->doImport(dirEnt.path(), asSetting, currentContent, task);
-
-        return getImportService(asSetting.adir)->getObject(dirEnt.path());
+        if (asSetting.changedObject || !asSetting.adir || asSetting.adir->getScanMode() != AutoscanScanMode::INotify) {
+            importService->clearCache(); // may be called by layout
+        }
+        return getImportService(asSetting.adir)->doImport(dirEnt.path(), asSetting, currentContent, task);
     }
     // checkDatabase, don't process existing
     std::shared_ptr<CdsContainer> parent;
-    auto parentObject = database->findObjectByPath(dirEnt.path().parent_path(), DbFileType::Directory);
+    auto parentObject = database->findObjectByPath(dirEnt.path().parent_path(), UNUSED_CLIENT_GROUP, DbFileType::Directory);
     if (parentObject && parentObject->isContainer())
         parent = std::dynamic_pointer_cast<CdsContainer>(parentObject);
     auto obj = createSingleItem(dirEnt, parent, rootPath, asSetting.followSymlinks, true, false, false, asSetting.adir, task);
@@ -486,7 +525,7 @@ bool ContentManager::updateAttachedResources(const std::shared_ptr<AutoscanDirec
         return false;
 
     bool parentRemoved = false;
-    auto parentObject = database->findObjectByPath(parentPath, DbFileType::Auto);
+    auto parentObject = database->findObjectByPath(parentPath, UNUSED_CLIENT_GROUP, DbFileType::Auto);
     if (parentObject) {
         // as there is no proper way to force refresh of unchanged files, delete whole dir and rescan it
         _removeObject(adir, parentObject, parentPath, false, all);
@@ -514,7 +553,12 @@ bool ContentManager::updateAttachedResources(const std::shared_ptr<AutoscanDirec
     return parentRemoved;
 }
 
-std::vector<int> ContentManager::_removeObject(const std::shared_ptr<AutoscanDirectory>& adir, const std::shared_ptr<CdsObject>& obj, const fs::path& path, bool rescanResource, bool all)
+std::vector<int> ContentManager::_removeObject(
+    const std::shared_ptr<AutoscanDirectory>& adir,
+    const std::shared_ptr<CdsObject>& obj,
+    const fs::path& path,
+    bool rescanResource,
+    bool all)
 {
     if (!obj || obj->getID() == INVALID_OBJECT_ID)
         return {};
@@ -681,6 +725,9 @@ void ContentManager::_rescanDirectory(const std::shared_ptr<AutoscanDirectory>&
     }
 
     if (importMode == ImportMode::Gerbera) {
+        if (asSetting.changedObject || !adir || adir->getScanMode() != AutoscanScanMode::INotify) {
+            importService->clearCache(); // may be called by layout
+        }
         getImportService(adir)->doImport(location, asSetting, list, task);
     } else {
         auto lastModifiedCurrentMax = adir->getPreviousLMT(location, parentContainer);
@@ -708,7 +755,7 @@ void ContentManager::_rescanDirectory(const std::shared_ptr<AutoscanDirectory>&
             }
 
             if (!asSetting.followSymlinks && dirEnt.is_symlink()) {
-                auto object = database->findObjectByPath(newPath);
+                auto object = database->findObjectByPath(newPath, UNUSED_CLIENT_GROUP);
                 if (object) {
                     list.erase(object->getID());
                     removeObject(adir, object, newPath, false);
@@ -724,7 +771,7 @@ void ContentManager::_rescanDirectory(const std::shared_ptr<AutoscanDirectory>&
             auto lwt = toSeconds(dirEnt.last_write_time(ec));
 
             if (isRegularFile(dirEnt, ec)) {
-                auto newObject = database->findObjectByPath(newPath);
+                auto newObject = database->findObjectByPath(newPath, UNUSED_CLIENT_GROUP);
                 if (newObject) {
                     list.erase(newObject->getID());
 
@@ -761,7 +808,7 @@ void ContentManager::_rescanDirectory(const std::shared_ptr<AutoscanDirectory>&
                     }
                 }
             } else if (dirEnt.is_directory(ec) && asSetting.recursive) {
-                auto newObject = database->findObjectByPath(newPath);
+                auto newObject = database->findObjectByPath(newPath, UNUSED_CLIENT_GROUP);
                 if (lastModifiedNewMax < lwt && lwt <= currentTme)
                     lastModifiedNewMax = lwt;
                 if (newObject) {
@@ -797,7 +844,7 @@ void ContentManager::_rescanDirectory(const std::shared_ptr<AutoscanDirectory>&
                 }
             }
             if (!parentContainer && !location.empty()) {
-                std::shared_ptr<CdsObject> obj = database->findObjectByPath(location, DbFileType::Directory);
+                std::shared_ptr<CdsObject> obj = database->findObjectByPath(location, UNUSED_CLIENT_GROUP, DbFileType::Directory);
                 if (!obj || !obj->isContainer()) {
                     log_error("Updated parent {} is not available or no container", location.string());
                 } else {
@@ -853,7 +900,7 @@ std::shared_ptr<AutoscanDirectory> ContentManager::findAutoscanDirectory(fs::pat
         if (dir) {
             log_debug("AutoscanDir ({}): {}", AutoscanDirectory::mapScanmode(dir->getScanMode()), i);
             if (isSubDir(path, dir->getLocation()) && fs::is_directory(dir->getLocation(), ec)) {
-                autoscanDir = std::move(dir);
+                autoscanDir = dir;
             }
             if (ec) {
                 log_warning("No AutoscanDir {} has problems '{}'", dir->getLocation().string(), ec.message());
@@ -876,7 +923,7 @@ void ContentManager::addRecursive(
     bool hidden,
     const std::shared_ptr<CMAddFileTask>& task)
 {
-    auto f2i = StringConverter::f2i(config);
+    auto f2i = converterManager->f2i();
 
     std::error_code ec;
     if (!subDir.exists(ec) || !subDir.is_directory(ec)) {
@@ -974,9 +1021,10 @@ void ContentManager::addRecursive(
 }
 
 template <typename T>
-void ContentManager::updateCdsObject(const std::shared_ptr<T>& item, const std::map<std::string, std::string>& parameters)
+std::shared_ptr<CdsObject> ContentManager::updateCdsObject(const std::shared_ptr<T>& item, const std::map<std::string, std::string>& parameters)
 {
     std::string title = getValueOrDefault(parameters, "title");
+    std::string sortKey = getValueOrDefault(parameters, "sortKey");
     std::string upnpClass = getValueOrDefault(parameters, "class");
     std::string mimetype = getValueOrDefault(parameters, "mime-type");
     std::string description = getValueOrDefault(parameters, "description");
@@ -985,12 +1033,14 @@ void ContentManager::updateCdsObject(const std::shared_ptr<T>& item, const std::
     std::string flags = getValueOrDefault(parameters, "flags");
 
     log_error("updateCdsObject: CdsObject {} not updated", title);
+    return nullptr;
 }
 
 template <>
-void ContentManager::updateCdsObject(const std::shared_ptr<CdsContainer>& item, const std::map<std::string, std::string>& parameters)
+std::shared_ptr<CdsObject> ContentManager::updateCdsObject(const std::shared_ptr<CdsContainer>& item, const std::map<std::string, std::string>& parameters)
 {
     std::string title = getValueOrDefault(parameters, "title");
+    std::string sortKey = getValueOrDefault(parameters, "sortKey");
     std::string upnpClass = getValueOrDefault(parameters, "class");
     std::string flags = getValueOrDefault(parameters, "flags");
 
@@ -1001,6 +1051,8 @@ void ContentManager::updateCdsObject(const std::shared_ptr<CdsContainer>& item,
 
     if (!title.empty())
         clone->setTitle(title);
+    if (!sortKey.empty())
+        clone->setSortKey(sortKey);
     if (!upnpClass.empty())
         clone->setClass(upnpClass);
     if (!flags.empty())
@@ -1019,16 +1071,18 @@ void ContentManager::updateCdsObject(const std::shared_ptr<CdsContainer>& item,
         update_manager->containerChanged(item->getParentID());
         session_manager->containerChangedUI(item->getParentID());
     }
+    return clone;
 }
 
 template <>
-void ContentManager::updateCdsObject(const std::shared_ptr<CdsItem>& item, const std::map<std::string, std::string>& parameters)
+std::shared_ptr<CdsObject> ContentManager::updateCdsObject(const std::shared_ptr<CdsItem>& item, const std::map<std::string, std::string>& parameters)
 {
     std::string title = getValueOrDefault(parameters, "title");
     std::string upnpClass = getValueOrDefault(parameters, "class");
     std::string mimetype = getValueOrDefault(parameters, "mime-type");
     std::string description = getValueOrDefault(parameters, "description");
     std::string location = getValueOrDefault(parameters, "location");
+    std::string sortKey = getValueOrDefault(parameters, "sortKey");
     std::string protocol = getValueOrDefault(parameters, "protocol");
     std::string flags = getValueOrDefault(parameters, "flags");
     log_debug("updateCdsObject: CdsItem {} updated", title);
@@ -1038,6 +1092,8 @@ void ContentManager::updateCdsObject(const std::shared_ptr<CdsItem>& item, const
 
     if (!title.empty())
         clone->setTitle(title);
+    if (!sortKey.empty())
+        clone->setSortKey(sortKey);
     if (!upnpClass.empty())
         clone->setClass(upnpClass);
     if (!location.empty())
@@ -1090,20 +1146,21 @@ void ContentManager::updateCdsObject(const std::shared_ptr<CdsItem>& item, const
         log_debug("updateObject: calling containerChanged on item {}", item->getTitle());
         update_manager->containerChanged(item->getParentID());
     }
+    return clone;
 }
 
-void ContentManager::updateObject(int objectID, const std::map<std::string, std::string>& parameters)
+std::shared_ptr<CdsObject> ContentManager::updateObject(int objectID, const std::map<std::string, std::string>& parameters)
 {
     auto obj = database->loadObject(objectID);
     auto item = std::dynamic_pointer_cast<CdsItem>(obj);
     if (item) {
-        updateCdsObject(item, parameters);
+        return updateCdsObject(item, parameters);
     } else {
         auto cont = std::dynamic_pointer_cast<CdsContainer>(obj);
         if (cont) {
-            updateCdsObject(cont, parameters);
+            return updateCdsObject(cont, parameters);
         } else {
-            updateCdsObject(obj, parameters);
+            return updateCdsObject(obj, parameters);
         }
     }
 }
@@ -1151,7 +1208,7 @@ std::shared_ptr<CdsContainer> ContentManager::addContainer(int parentID, const s
         cVec.push_back(std::make_shared<CdsContainer>(segment.string(), upnpClass));
     }
     addContainerTree(cVec, nullptr);
-    return importService->getContainer(cPath);
+    return importService->getContainer(fmt::format("/{}", cPath.string()));
 }
 
 std::pair<int, bool> ContentManager::addContainerTree(const std::vector<std::shared_ptr<CdsObject>>& chain, const std::shared_ptr<CdsObject>& refItem)
@@ -1224,12 +1281,12 @@ void ContentManager::initLayout()
     if (layoutEnabled) {
         auto self = shared_from_this();
         auto lock = threadRunner->lockGuard("initLayout");
-        auto layoutType = EnumOption<LayoutType>::getEnumOption(config, CFG_IMPORT_SCRIPTING_VIRTUAL_LAYOUT_TYPE);
+        auto layoutType = EnumOption<LayoutType>::getEnumOption(config, ConfigVal::IMPORT_SCRIPTING_VIRTUAL_LAYOUT_TYPE);
         importService->initLayout(layoutType);
         for (std::size_t i = 0; i < autoscanList->size(); i++) {
             auto autoscanDir = autoscanList->get(i);
 
-            auto asImportService = std::make_shared<ImportService>(context);
+            auto asImportService = std::make_shared<ImportService>(context, converterManager);
             asImportService->run(self, autoscanDir, autoscanDir->getLocation());
             autoscanDir->setImportService(asImportService);
             asImportService->initLayout(layoutType);
@@ -1313,7 +1370,11 @@ void ContentManager::addTask(std::shared_ptr<GenericTask> task, bool lowPriority
     threadRunner->notify();
 }
 
-std::shared_ptr<CdsObject> ContentManager::addFile(const fs::directory_entry& dirEnt, AutoScanSetting& asSetting, bool lowPriority, bool cancellable)
+std::shared_ptr<CdsObject> ContentManager::addFile(
+    const fs::directory_entry& dirEnt,
+    AutoScanSetting& asSetting,
+    bool lowPriority,
+    bool cancellable)
 {
     fs::path rootpath;
     if (dirEnt.is_directory())
@@ -1321,13 +1382,23 @@ std::shared_ptr<CdsObject> ContentManager::addFile(const fs::directory_entry& di
     return addFileInternal(dirEnt, rootpath, asSetting, lowPriority, 0, cancellable);
 }
 
-std::shared_ptr<CdsObject> ContentManager::addFile(const fs::directory_entry& dirEnt, const fs::path& rootpath, AutoScanSetting& asSetting, bool lowPriority, bool cancellable)
+std::shared_ptr<CdsObject> ContentManager::addFile(
+    const fs::directory_entry& dirEnt,
+    const fs::path& rootpath,
+    AutoScanSetting& asSetting,
+    bool lowPriority,
+    bool cancellable)
 {
     return addFileInternal(dirEnt, rootpath, asSetting, lowPriority, 0, cancellable);
 }
 
 std::shared_ptr<CdsObject> ContentManager::addFileInternal(
-    const fs::directory_entry& dirEnt, const fs::path& rootpath, AutoScanSetting& asSetting, bool lowPriority, unsigned int parentTaskID, bool cancellable)
+    const fs::directory_entry& dirEnt,
+    const fs::path& rootpath,
+    AutoScanSetting& asSetting,
+    bool lowPriority,
+    unsigned int parentTaskID,
+    bool cancellable)
 {
     if (asSetting.async) {
         auto self = shared_from_this();
@@ -1393,7 +1464,7 @@ void ContentManager::cleanupOnlineServiceObjects(const std::shared_ptr<OnlineSer
 
 void ContentManager::invalidateAddTask(const std::shared_ptr<GenericTask>& t, const fs::path& path)
 {
-    if (t->getType() == AddFile) {
+    if (t->getType() == TaskType::AddFile) {
         auto addTask = std::static_pointer_cast<CMAddFileTask>(t);
         log_debug("comparing, task path: {}, remove path: {}", addTask->getPath().c_str(), path.c_str());
         if (isSubDir(addTask->getPath(), path)) {
@@ -1403,9 +1474,9 @@ void ContentManager::invalidateAddTask(const std::shared_ptr<GenericTask>& t, co
     }
 }
 
-void ContentManager::invalidateTask(unsigned int taskID, task_owner_t taskOwner)
+void ContentManager::invalidateTask(unsigned int taskID, TaskOwner taskOwner)
 {
-    if (taskOwner == ContentManagerTask) {
+    if (taskOwner == TaskOwner::ContentManagerTask) {
         auto lock = threadRunner->lockGuard("invalidateTask");
         auto tc = getCurrentTask();
         if (tc && ((tc->getID() == taskID) || (tc->getParentID() == taskID)))
@@ -1422,12 +1493,18 @@ void ContentManager::invalidateTask(unsigned int taskID, task_owner_t taskOwner)
         }
     }
 #ifdef ONLINE_SERVICES
-    else if (taskOwner == TaskProcessorTask)
+    else if (taskOwner == TaskOwner::TaskProcessorTask)
         task_processor->invalidateTask(taskID);
 #endif
 }
 
-std::vector<int> ContentManager::removeObject(const std::shared_ptr<AutoscanDirectory>& adir, const std::shared_ptr<CdsObject>& obj, const fs::path& path, bool rescanResource, bool async, bool all)
+std::vector<int> ContentManager::removeObject(
+    const std::shared_ptr<AutoscanDirectory>& adir,
+    const std::shared_ptr<CdsObject>& obj,
+    const fs::path& path,
+    bool rescanResource,
+    bool async,
+    bool all)
 {
     if (async) {
         auto self = shared_from_this();
@@ -1455,8 +1532,8 @@ void ContentManager::cleanupTasks(const fs::path& path)
         if (dir->getScanMode() == AutoscanScanMode::Timed) {
             timer->removeTimerSubscriber(this, rmList->get(i)->getTimerParameter(), true);
 #ifdef HAVE_INOTIFY
-        } else if (config->getBoolOption(CFG_IMPORT_AUTOSCAN_USE_INOTIFY)) {
-            inotify->unmonitor(dir);
+        } else if (useAsInotify) {
+            as_inotify->unmonitor(dir);
 #endif
         }
     }
@@ -1479,7 +1556,11 @@ void ContentManager::cleanupTasks(const fs::path& path)
     }
 }
 
-void ContentManager::rescanDirectory(const std::shared_ptr<AutoscanDirectory>& adir, int objectId, fs::path descPath, bool cancellable)
+void ContentManager::rescanDirectory(
+    const std::shared_ptr<AutoscanDirectory>& adir,
+    int objectId,
+    fs::path descPath,
+    bool cancellable)
 {
     auto self = shared_from_this();
     auto task = std::make_shared<CMRescanDirectoryTask>(self, adir, objectId, cancellable);
@@ -1506,7 +1587,7 @@ std::shared_ptr<AutoscanDirectory> ContentManager::getAutoscanDirectory(int obje
 
 std::shared_ptr<AutoscanDirectory> ContentManager::getAutoscanDirectory(const fs::path& location) const
 {
-    return autoscanList->get(location);
+    return autoscanList->getKey(location);
 }
 
 std::vector<std::shared_ptr<AutoscanDirectory>> ContentManager::getAutoscanDirectories() const
@@ -1520,7 +1601,7 @@ void ContentManager::removeAutoscanDirectory(const std::shared_ptr<AutoscanDirec
         throw_std_runtime_error("can not remove autoscan directory - was not an autoscan");
 
     adir->setTaskCount(-1);
-    autoscanList->remove(adir->getScanID());
+    autoscanList->remove(adir);
     database->removeAutoscanDirectory(adir);
     session_manager->containerChangedUI(adir->getObjectID());
 
@@ -1529,8 +1610,8 @@ void ContentManager::removeAutoscanDirectory(const std::shared_ptr<AutoscanDirec
         timer->removeTimerSubscriber(this, adir->getTimerParameter(), true);
     }
 #ifdef HAVE_INOTIFY
-    else if (config->getBoolOption(CFG_IMPORT_AUTOSCAN_USE_INOTIFY) && (adir->getScanMode() == AutoscanScanMode::INotify)) {
-        inotify->unmonitor(adir);
+    else if (useAsInotify && (adir->getScanMode() == AutoscanScanMode::INotify)) {
+        as_inotify->unmonitor(adir);
     }
 #endif
 }
@@ -1582,10 +1663,10 @@ void ContentManager::setAutoscanDirectory(const std::shared_ptr<AutoscanDirector
         dir->resetLMT();
         database->addAutoscanDirectory(dir);
         auto self = shared_from_this();
-        auto asImportService = std::make_shared<ImportService>(context);
+        auto asImportService = std::make_shared<ImportService>(context, converterManager);
         asImportService->run(self, dir, dir->getLocation());
         dir->setImportService(asImportService);
-        auto layoutType = EnumOption<LayoutType>::getEnumOption(config, CFG_IMPORT_SCRIPTING_VIRTUAL_LAYOUT_TYPE);
+        auto layoutType = EnumOption<LayoutType>::getEnumOption(config, ConfigVal::IMPORT_SCRIPTING_VIRTUAL_LAYOUT_TYPE);
         asImportService->initLayout(layoutType);
         scanDir(dir, true);
         return;
@@ -1594,8 +1675,8 @@ void ContentManager::setAutoscanDirectory(const std::shared_ptr<AutoscanDirector
     if (original->getScanMode() == AutoscanScanMode::Timed)
         timer->removeTimerSubscriber(this, original->getTimerParameter(), true);
 #ifdef HAVE_INOTIFY
-    else if (config->getBoolOption(CFG_IMPORT_AUTOSCAN_USE_INOTIFY) && (original->getScanMode() == AutoscanScanMode::INotify))
-        inotify->unmonitor(original);
+    else if (useAsInotify && (original->getScanMode() == AutoscanScanMode::INotify))
+        as_inotify->unmonitor(original);
 #endif
 
     auto copy = std::make_shared<AutoscanDirectory>();
@@ -1605,23 +1686,28 @@ void ContentManager::setAutoscanDirectory(const std::shared_ptr<AutoscanDirector
     copy->setRecursive(dir->getRecursive());
     copy->setInterval(dir->getInterval());
     copy->setScanContent(dir->getScanContent());
+    copy->setDirTypes(dir->hasDirTypes());
 
-    autoscanList->remove(copy->getScanID());
+    if (dir->isValid())
+        autoscanList->remove(dir); // remove old version of dir
 
-    copy->setScanMode(dir->getScanMode());
     scanDir(copy, original->getScanMode() != copy->getScanMode());
+    copy->setScanMode(dir->getScanMode());
     database->updateAutoscanDirectory(copy);
 }
 
 void ContentManager::scanDir(const std::shared_ptr<AutoscanDirectory>& dir, bool updateUI)
 {
-    autoscanList->add(dir);
+    if (dir->isValid())
+        autoscanList->add(dir, dir->getScanID());
+    else
+        autoscanList->add(dir);
 
     if (dir->getScanMode() == AutoscanScanMode::Timed)
         timerNotify(dir->getTimerParameter());
 #ifdef HAVE_INOTIFY
-    else if (config->getBoolOption(CFG_IMPORT_AUTOSCAN_USE_INOTIFY) && (dir->getScanMode() == AutoscanScanMode::INotify))
-        inotify->monitor(dir);
+    else if (useAsInotify && (dir->getScanMode() == AutoscanScanMode::INotify))
+        as_inotify->monitor(dir);
 #endif
 
     if (updateUI)
@@ -1642,13 +1728,13 @@ void ContentManager::triggerPlayHook(const std::string& group, const std::shared
     playStatus->setLastPlayed();
     database->savePlayStatus(playStatus);
 
-    bool suppress = config->getBoolOption(CFG_SERVER_EXTOPTS_MARK_PLAYED_ITEMS_SUPPRESS_CDS_UPDATES);
+    bool suppress = config->getBoolOption(ConfigVal::SERVER_EXTOPTS_MARK_PLAYED_ITEMS_ENABLED) && config->getBoolOption(ConfigVal::SERVER_EXTOPTS_MARK_PLAYED_ITEMS_SUPPRESS_CDS_UPDATES);
     log_debug("Marking object {} as played", obj->getTitle());
     if (!suppress)
         updateObject(obj, true);
 
-#ifdef HAVE_LASTFMLIB
-    if (config->getBoolOption(CFG_SERVER_EXTOPTS_LASTFM_ENABLED) && item->isSubClass(UPNP_CLASS_AUDIO_ITEM)) {
+#ifdef HAVE_LASTFM
+    if (config->getBoolOption(ConfigVal::SERVER_EXTOPTS_LASTFM_ENABLED) && item->isSubClass(UPNP_CLASS_AUDIO_ITEM)) {
         last_fm->startedPlaying(item);
     }
 #endif
