@@ -4,7 +4,7 @@
 
     context.cc - this file is part of Gerbera.
 
-    Copyright (C) 2021-2024 Gerbera Contributors
+    Copyright (C) 2021-2025 Gerbera Contributors
 
     Gerbera is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -21,19 +21,24 @@
     $Id$
 */
 
-/// \file context.cc
-
+/// @file context.cc
+#define GRB_LOG_FAC GrbLogFacility::content
 #include "context.h" // API
 
-Context::Context(std::shared_ptr<Config> config,
+Context::Context(
+    std::shared_ptr<ConfigDefinition> definition,
+    std::shared_ptr<Config> config,
     std::shared_ptr<ClientManager> clients,
     std::shared_ptr<Mime> mime,
     std::shared_ptr<Database> database,
-    std::shared_ptr<Web::SessionManager> sessionManager)
-    : config(std::move(config))
+    std::shared_ptr<Web::SessionManager> sessionManager,
+    std::shared_ptr<ConverterManager> converterManager)
+    : definition(std::move(definition))
+    , config(std::move(config))
     , clients(std::move(clients))
     , mime(std::move(mime))
     , database(std::move(database))
     , session_manager(std::move(sessionManager))
+    , converterManager(std::move(converterManager))
 {
 }
