@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    sqlite3_database.cc - this file is part of MediaTomb.
+    sqlite_database.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -29,50 +29,153 @@
     $Id$
 */
 
-/// \file sqlite3_database.cc
-#define LOG_FAC log_facility_t::sqlite3
+/// @file database/sqlite3/sqlite_database.cc
+#define GRB_LOG_FAC GrbLogFacility::sqlite3
 
 #include "sqlite_database.h" // API
 
-#include "config/config_manager.h"
+#include "cds/cds_enums.h"
+#include "config/config.h"
+#include "config/config_val.h"
+#include "exceptions.h"
+#include "sl_result.h"
 #include "sl_task.h"
 
+#include <sqlite3.h>
+
 static constexpr auto sqlite3UpdateVersion = std::string_view(R"(UPDATE "mt_internal_setting" SET "value"='{}' WHERE "key"='db_version' AND "value"='{}')");
-static constexpr auto sqlite3AddResourceAttr = std::string_view(R"(ALTER TABLE "grb_cds_resource" ADD COLUMN "{}" varchar(255) default NULL)");
+static const auto sqlite3AddResourceAttr = std::map<ResourceDataType, std::string_view> {
+    { ResourceDataType::String, R"(ALTER TABLE "grb_cds_resource" ADD COLUMN "{}" varchar(255) default NULL)" },
+    { ResourceDataType::Number, R"(ALTER TABLE "grb_cds_resource" ADD COLUMN "{}" bigint default NULL)" }
+};
 
 #define DELETE_CACHE_MAX_TIME 60 // drop cache if last delete was more than 60 secs ago
 #define DELETE_CACHE_RED_SIZE 0.2 // reduce cache to 80% of max entries
 
-Sqlite3Database::Sqlite3Database(const std::shared_ptr<Config>& config, const std::shared_ptr<Mime>& mime, std::shared_ptr<Timer> timer)
-    : SQLDatabase(config, mime)
+Sqlite3Database::Sqlite3Database(const std::shared_ptr<Config>& config, const std::shared_ptr<Mime>& mime, const std::shared_ptr<ConverterManager>& converterManager, std::shared_ptr<Timer> timer)
+    : SQLDatabase(config, mime, converterManager)
     , timer(std::move(timer))
+    , shutdownAttempts(this->config->getIntOption(ConfigVal::SERVER_STORAGE_SQLITE_SHUTDOWN_ATTEMPTS))
 {
+    dbFilePath = config->getOption(ConfigVal::SERVER_STORAGE_SQLITE_DATABASE_FILE);
     table_quote_begin = '"';
     table_quote_end = '"';
 
     // if sqlite3.sql or sqlite3-upgrade.xml is changed hashies have to be updated
-    hashies = { 220562690, // index 0 is used for create script sqlite3.sql = Version 1
-        778996897, 3362507034, 853149842, 4035419264, 3497064885, 974692115, 119767663, 3167732653, 2427825904, 3305506356, // upgrade 2-11
-        43189396, 2767540493, 2512852146, 1273710965, 319062951, 3593597366, 1028160353, 881071639, 1989518047, 3743992560, // upgrade 12-21
-        3135921396 };
+    hashies = {
+        3418613970, // index 0 is used for create script sqlite3.sql = Version 1
+        778996897, 3362507034, 853149842, 2776802417, 3497064885, 974692115, 119767663, 3167732653, 2427825904, 3305506356, // upgrade 2-11
+        3908767237, 509765404, 2512852146, 1273710965, 319062951, 2316641127, 1028160353, 881071639, 1989518047, 782849313, // upgrade 12-21
+        3135921396, 3108208, 2156790525, 2004941040,
+        459854332 // index DBVERSION is used for drop script sqlite3-drop.sql = Version -1
+    };
 }
 
 void Sqlite3Database::prepare()
 {
     _exec("PRAGMA locking_mode = EXCLUSIVE");
     _exec("PRAGMA foreign_keys = ON");
-    _exec(fmt::format("PRAGMA journal_mode = {}", config->getOption(CFG_SERVER_STORAGE_SQLITE_JOURNALMODE)));
-    exec(fmt::format("PRAGMA synchronous = {}", config->getIntOption(CFG_SERVER_STORAGE_SQLITE_SYNCHRONOUS)));
+    _exec(fmt::format("PRAGMA journal_mode = {}", config->getOption(ConfigVal::SERVER_STORAGE_SQLITE_JOURNALMODE)));
+    exec(fmt::format("PRAGMA synchronous = {}", config->getIntOption(ConfigVal::SERVER_STORAGE_SQLITE_SYNCHRONOUS)));
 }
 
-void Sqlite3Database::init()
+std::string Sqlite3Database::prepareDatabase(const fs::path& dbFilePath, GrbFile& dbFile)
 {
-    dbInitDone = false;
-    SQLDatabase::init();
+    std::string dbVersion;
+    fs::path dbFilePathbackup = fmt::format(SQLITE3_BACKUP_FORMAT, dbFilePath.c_str());
+    auto dbBackupFile = GrbFile(dbFilePathbackup);
+    try {
+        dbVersion = getInternalSetting("db_version");
+    } catch (const std::runtime_error&) {
+        log_warning("SQLite3 database seems to be corrupt or doesn't exist yet.");
+        // database seems to be corrupt or nonexistent
+        if (config->getBoolOption(ConfigVal::SERVER_STORAGE_SQLITE_RESTORE) && sqliteStatus == SQLITE_OK) {
+            // try to restore database
 
-    fs::path dbFilePath = config->getOption(CFG_SERVER_STORAGE_SQLITE_DATABASE_FILE);
+            // checking for backup file
+            if (dbBackupFile.isReadable()) {
+                try {
+                    // trying to copy backup file
+                    auto btask = std::make_shared<SLBackupTask>(config, true);
+                    this->addTask(btask);
+                    btask->waitForTask();
+                    prepare();
+                    dbVersion = getInternalSetting("db_version");
+                } catch (const std::runtime_error&) {
+                }
+            }
+        } else {
+            // fail because restore option is false
+            shutdown();
+            throw_std_runtime_error("SQLite3 database seems to be corrupt and the 'on-error' option is set to 'fail'");
+        }
+    }
+
+    if (dbVersion.empty() && !dbBackupFile.isReadable()) {
+        log_info("No SQLite3 backup is available or backup is corrupt. Automatically creating new database file...");
+        auto itask = std::make_shared<SLScriptTask>(config, hashies.at(0), stringLimit, ConfigVal::SERVER_STORAGE_SQLITE_INIT_SQL_FILE);
+        addTask(itask);
+        try {
+            itask->waitForTask();
+            fillDatabase();
+            storeInternalSetting("string_limit", fmt::to_string(stringLimit));
+            storeInternalSetting("resource_attribute", "");
+            prepare();
+            dbFile.setPermissions();
+            dbVersion = getInternalSetting("db_version");
+        } catch (const std::runtime_error& e) {
+            shutdown();
+            throw_std_runtime_error("Error while creating database: {}", e.what());
+        }
+        log_info("Database tables {} created successfully.", dbFilePath.c_str());
+    }
+
+    if (dbVersion.empty()) {
+        shutdown();
+        throw_std_runtime_error("SQLite3 database seems to be corrupt and restoring from backup failed");
+    }
+
+    return dbVersion;
+}
+
+void Sqlite3Database::dropTables()
+{
+    auto file = config->getOption(ConfigVal::SERVER_STORAGE_SQLITE_DROP_FILE);
+    log_info("Dropping tables with {}", file);
+    auto dtask = std::make_shared<SLScriptTask>(config, hashies.at(DBVERSION), stringLimit, ConfigVal::SERVER_STORAGE_SQLITE_DROP_FILE);
+    addTask(dtask);
+    try {
+        dtask->waitForTask();
+    } catch (const std::runtime_error& e) {
+        shutdown();
+        throw_std_runtime_error("Error while dropping database: {}", e.what());
+    }
+
+    // back up file to make sure the tables are dropped from backup as well
+    if (config->getBoolOption(ConfigVal::SERVER_STORAGE_SQLITE_RESTORE) && config->getBoolOption(ConfigVal::SERVER_STORAGE_SQLITE_BACKUP_ENABLED)) {
+        fs::path dbFilePathbackup = fmt::format(SQLITE3_BACKUP_FORMAT, dbFilePath.c_str());
+        auto dbBackupFile = GrbFile(dbFilePathbackup);
+
+        // checking for backup file
+        if (dbBackupFile.isReadable()) {
+            try {
+                auto btask = std::make_shared<SLBackupTask>(config, false);
+                this->addTask(btask);
+                btask->waitForTask();
+            } catch (const std::runtime_error& e) {
+                shutdown();
+                throw_std_runtime_error("Error while backing up database: {}", e.what());
+            }
+        }
+    }
+
+    log_info("Database tables {} dropped successfully.", dbFilePath.c_str());
+}
+
+void Sqlite3Database::run()
+{
     auto dbFile = GrbFile(dbFilePath);
-    log_debug("SQLite path: {}", dbFilePath.c_str());
+    log_debug("SQLite3 path: {}", dbFilePath.c_str());
 
     // check for db-file
     if (!dbFile.isWritable())
@@ -98,76 +201,50 @@ void Sqlite3Database::init()
         prepare();
     } catch (const std::runtime_error&) {
         shutdown();
-        throw_std_runtime_error("Sqlite3Database.init: could not open '{}' exclusively", dbFilePath.c_str());
+        throw_std_runtime_error("Sqlite3Database.run: could not open '{}' exclusively", dbFilePath.c_str());
     }
+}
 
-    std::string dbVersion;
-    fs::path dbFilePathbackup = fmt::format(SQLITE3_BACKUP_FORMAT, dbFilePath.c_str());
-    auto dbBackupFile = GrbFile(dbFilePathbackup);
-    try {
-        dbVersion = getInternalSetting("db_version");
-    } catch (const std::runtime_error&) {
-        log_warning("Sqlite3 database seems to be corrupt or doesn't exist yet.");
-        // database seems to be corrupt or nonexistent
-        if (config->getBoolOption(CFG_SERVER_STORAGE_SQLITE_RESTORE) && sqliteStatus == SQLITE_OK) {
-            // try to restore database
+void Sqlite3Database::init()
+{
+    dbInitDone = false;
+    SQLDatabase::init();
 
-            // checking for backup file
-            if (dbBackupFile.isReadable()) {
-                try {
-                    // trying to copy backup file
-                    auto btask = std::make_shared<SLBackupTask>(config, true);
-                    this->addTask(btask);
-                    btask->waitForTask();
-                    prepare();
-                    dbVersion = getInternalSetting("db_version");
-                } catch (const std::runtime_error&) {
-                }
-            }
-        } else {
-            // fail because restore option is false
-            shutdown();
-            throw_std_runtime_error("sqlite3 database seems to be corrupt and the 'on-error' option is set to 'fail'");
-        }
-    }
-
-    if (dbVersion.empty() && !dbBackupFile.isReadable()) {
-        log_info("No sqlite3 backup is available or backup is corrupt. Automatically creating new database file...");
-        auto itask = std::make_shared<SLInitTask>(config, hashies[0]);
-        addTask(itask);
-        try {
-            itask->waitForTask();
-            prepare();
-            dbFile.setPermissions();
-            dbVersion = getInternalSetting("db_version");
-        } catch (const std::runtime_error& e) {
-            shutdown();
-            throw_std_runtime_error("Error while creating database: {}", e.what());
-        }
-        log_info("database created successfully.");
-    }
-
-    if (dbVersion.empty()) {
-        shutdown();
-        throw_std_runtime_error("sqlite3 database seems to be corrupt and restoring from backup failed");
-    }
+    auto dbFile = GrbFile(dbFilePath);
+    std::string dbVersion = prepareDatabase(dbFilePath, dbFile);
 
     try {
-        upgradeDatabase(std::stoul(dbVersion), hashies, CFG_SERVER_STORAGE_SQLITE_UPGRADE_FILE, sqlite3UpdateVersion, sqlite3AddResourceAttr);
-        if (config->getBoolOption(CFG_SERVER_STORAGE_SQLITE_BACKUP_ENABLED) && timer) {
+        upgradeDatabase(std::stoul(dbVersion), hashies, ConfigVal::SERVER_STORAGE_SQLITE_UPGRADE_FILE, sqlite3UpdateVersion, sqlite3AddResourceAttr);
+        if (config->getBoolOption(ConfigVal::SERVER_STORAGE_SQLITE_BACKUP_ENABLED) && timer) {
             // do a backup now
             auto btask = std::make_shared<SLBackupTask>(config, false);
             this->addTask(btask);
             btask->waitForTask();
 
             // add timer for backups
-            auto backupInterval = std::chrono::seconds(config->getIntOption(CFG_SERVER_STORAGE_SQLITE_BACKUP_INTERVAL));
+            auto backupInterval = std::chrono::seconds(config->getLongOption(ConfigVal::SERVER_STORAGE_SQLITE_BACKUP_INTERVAL));
             timer->addTimerSubscriber(this, backupInterval, nullptr);
             hasBackupTimer = true;
         }
+
+        auto dbLimit = getInternalSetting("string_limit");
+        if (!dbLimit.empty() && std::stoul(dbLimit) > stringLimit)
+            log_warning("Your database was created with string length {} but your current config value is {}", dbLimit, stringLimit);
+        else if (dbLimit.empty()) {
+            log_info("Saving string limit {}", stringLimit);
+            storeInternalSetting("string_limit", fmt::to_string(stringLimit));
+        }
         dbInitDone = true;
     } catch (const std::runtime_error& e) {
-        log_error("prematurely shutting down.");
+        log_error("Prematurely shutting down.");
+        shutdown();
+        throw_std_runtime_error(e.what());
+    }
+
+    try {
+        initDynContainers();
+    } catch (const std::runtime_error& e) {
+        log_error("Prematurely shutting down because of initDynContainers.");
         shutdown();
         throw_std_runtime_error(e.what());
     }
@@ -180,7 +257,7 @@ std::shared_ptr<Database> Sqlite3Database::getSelf()
 
 void Sqlite3Database::_exec(const std::string& query)
 {
-    exec(query, false);
+    execOnly(query);
 }
 
 std::string Sqlite3Database::quote(const std::string& value) const
@@ -197,43 +274,24 @@ std::string Sqlite3Database::quote(const std::string& value) const
 std::string Sqlite3Database::handleError(const std::string& query, const std::string& error, sqlite3* db, int errorCode)
 {
     sqliteStatus = (errorCode == SQLITE_BUSY || errorCode == SQLITE_LOCKED) ? SQLITE_LOCKED : SQLITE_OK;
-    return fmt::format("SQLITE3 ({}: {}): {}\n       Query: {}\n       Error: {}", sqlite3_errcode(db), sqlite3_extended_errcode(db), sqlite3_errmsg(db), query.empty() ? "unknown" : query, error.empty() ? "unknown" : error);
+    return fmt::format("SQLite3 ({}: {}): {}\n       Query: {}\n       Error: {}", sqlite3_errcode(db), sqlite3_extended_errcode(db), sqlite3_errmsg(db), query.empty() ? "unknown" : query, error.empty() ? "unknown" : error);
 }
 
-Sqlite3DatabaseWithTransactions::Sqlite3DatabaseWithTransactions(const std::shared_ptr<Config>& config, const std::shared_ptr<Mime>& mime, const std::shared_ptr<Timer>& timer)
-    : SqlWithTransactions(config)
-    , Sqlite3Database(config, mime, timer)
+void Sqlite3Database::handleException(const std::exception& exc, const std::string& lineMessage)
 {
-}
-
-void Sqlite3DatabaseWithTransactions::beginTransaction(std::string_view tName)
-{
-    if (use_transaction) {
-        log_debug("BEGIN TRANSACTION {} {}", tName, inTransaction);
-        SqlAutoLock lock(sqlMutex);
-        StdThreadRunner::waitFor(
-            fmt::format("SqliteDatabase.begin {}", tName), [this] { return !inTransaction; }, 100);
-        inTransaction = true;
-        _exec("BEGIN TRANSACTION");
+    if (!dbInitDone)
+        throw_std_runtime_error(exc.what());
+    ++shutdownFlag;
+    if (shutdownFlag <= 1) {
+        log_error("Prematurely shutting down.\n{}\n{}", lineMessage, exc.what());
+        rollback("");
+        shutdown();
     }
-}
-
-void Sqlite3DatabaseWithTransactions::rollback(std::string_view tName)
-{
-    if (use_transaction && inTransaction) {
-        log_debug("ROLLBACK {} {}", tName, inTransaction);
-        _exec("ROLLBACK");
-        inTransaction = false;
-    }
-}
-
-void Sqlite3DatabaseWithTransactions::commit(std::string_view tName)
-{
-    if (use_transaction && inTransaction) {
-        log_debug("COMMIT {} {}", tName, inTransaction);
-        _exec("COMMIT");
-        inTransaction = false;
+    if (shutdownFlag >= shutdownAttempts) {
+        log_error("Exceeding shutdown limit {}.\n{}\n{}", shutdownAttempts, lineMessage, exc.what());
+        std::exit(1);
     }
+    log_error("Already shutting down.\n{}\n{}", lineMessage, exc.what());
 }
 
 std::shared_ptr<SQLResult> Sqlite3Database::select(const std::string& query)
@@ -245,14 +303,7 @@ std::shared_ptr<SQLResult> Sqlite3Database::select(const std::string& query)
         stask->waitForTask();
         return stask->getResult();
     } catch (const std::runtime_error& e) {
-        if (!dbInitDone)
-            throw_std_runtime_error(e.what());
-        if (!shutdownFlag) {
-            log_error("Prematurely shutting down.\n{}", e.what());
-            rollback("");
-            shutdown();
-        }
-        log_error("Already shutting down.\n{}", e.what());
+        handleException(e, LINE_MESSAGE);
         return {};
     }
 }
@@ -272,22 +323,15 @@ void Sqlite3Database::del(std::string_view tableName, const std::string& clause,
             }
             lastDelete = currentTime();
         }
-        auto etask = std::make_shared<SLExecTask>(query, false);
+        auto etask = std::make_shared<SLExecTask>(query, "", false);
         addTask(etask);
         etask->waitForTask();
     } catch (const std::runtime_error& e) {
-        if (!dbInitDone)
-            throw_std_runtime_error(e.what());
-        if (!shutdownFlag) {
-            log_error("Prematurely shutting down.\n{}", e.what());
-            rollback("");
-            shutdown();
-        }
-        log_error("Already shutting down.\n{}", e.what());
+        handleException(e, LINE_MESSAGE);
     }
 }
 
-void Sqlite3Database::exec(std::string_view tableName, const std::string& query, int objId)
+void Sqlite3Database::execOnTable(std::string_view tableName, const std::string& query, int objId)
 {
     try {
         log_debug("Adding query to Queue: {}", query);
@@ -296,34 +340,20 @@ void Sqlite3Database::exec(std::string_view tableName, const std::string& query,
         addTask(etask);
         etask->waitForTask();
     } catch (const std::runtime_error& e) {
-        if (!dbInitDone)
-            throw_std_runtime_error(e.what());
-        if (!shutdownFlag) {
-            log_error("Prematurely shutting down.\n{}", e.what());
-            rollback("");
-            shutdown();
-        }
-        log_error("Already shutting down.\n{}", e.what());
+        handleException(e, LINE_MESSAGE);
     }
 }
 
-int Sqlite3Database::exec(const std::string& query, bool getLastInsertId)
+int Sqlite3Database::exec(const std::string& query, const std::string& getLastInsertId)
 {
     try {
         log_debug("Adding query to Queue: {}", query);
-        auto etask = std::make_shared<SLExecTask>(query, getLastInsertId);
+        auto etask = std::make_shared<SLExecTask>(query, getLastInsertId, false);
         addTask(etask);
         etask->waitForTask();
-        return getLastInsertId ? etask->getLastInsertId() : -1;
+        return getLastInsertId.empty() ? -1 : etask->getLastInsertId();
     } catch (const std::runtime_error& e) {
-        if (!dbInitDone)
-            throw_std_runtime_error(e.what());
-        if (!shutdownFlag) {
-            log_error("Prematurely shutting down.\n{}", e.what());
-            rollback("");
-            shutdown();
-        }
-        log_error("Already shutting down.\n{}", e.what());
+        handleException(e, LINE_MESSAGE);
         return -1;
     }
 }
@@ -332,7 +362,7 @@ void Sqlite3Database::execOnly(const std::string& query)
 {
     try {
         log_debug("Adding query to Queue: {}", query);
-        auto etask = std::make_shared<SLExecTask>(query, false, false);
+        auto etask = std::make_shared<SLExecTask>(query, "", false);
         addTask(etask);
         etask->waitForTask();
     } catch (const std::runtime_error& e) {
@@ -346,11 +376,9 @@ void Sqlite3Database::threadProc()
     try {
         sqlite3* db;
 
-        std::string dbFilePath = config->getOption(CFG_SERVER_STORAGE_SQLITE_DATABASE_FILE);
-
         int res = sqlite3_open(dbFilePath.c_str(), &db);
         if (res != SQLITE_OK) {
-            startupError = fmt::format("Sqlite3Database.init: could not open '{}'", dbFilePath);
+            startupError = fmt::format("Sqlite3Database.threadProc: could not open '{}'", dbFilePath.c_str());
             return;
         }
 
@@ -374,7 +402,7 @@ void Sqlite3Database::threadProc()
 
                 lock.unlock();
                 try {
-                    task->run(db, this, throwOnError(task));
+                    task->run(db, *this, throwOnError(task));
                     if (task->didContamination())
                         dirty = true;
                     else if (task->didDecontamination())
@@ -382,6 +410,8 @@ void Sqlite3Database::threadProc()
                     task->sendSignal();
                 } catch (const std::runtime_error& e) {
                     task->sendSignal(e.what());
+                } catch (const std::logic_error& e) {
+                    task->sendSignal(e.what());
                 }
                 lock.lock();
             }
@@ -404,7 +434,7 @@ void Sqlite3Database::threadProc()
         while (!taskQueue.empty()) {
             auto task = std::move(taskQueue.front());
             taskQueue.pop();
-            task->sendSignal("Sorry, sqlite3 thread is shutting down");
+            task->sendSignal("Sorry, SQLite3 thread is shutting down");
         }
 
         if (db) {
@@ -432,13 +462,13 @@ void Sqlite3Database::timerNotify(const std::shared_ptr<Timer::Parameter>& param
 void Sqlite3Database::addTask(const std::shared_ptr<SLTask>& task, bool onlyIfDirty)
 {
     if (!taskQueueOpen) {
-        throw_std_runtime_error("sqlite3 task queue is already closed");
+        throw_std_runtime_error("SQLite3 task queue is already closed");
     }
 
     auto lock = threadRunner->lockGuard(fmt::format("addTask {}", task->taskType()));
 
     if (!taskQueueOpen) {
-        throw_std_runtime_error("sqlite3 task queue is already closed");
+        throw_std_runtime_error("SQLite3 task queue is already closed");
     }
     if (!onlyIfDirty || dirty) {
         taskQueue.push(task);
@@ -468,39 +498,48 @@ void Sqlite3Database::storeInternalSetting(const std::string& key, const std::st
 {
     auto command = fmt::format("INSERT OR REPLACE INTO {} ({}, {}) VALUES ({}, {})",
         identifier(INTERNAL_SETTINGS_TABLE), identifier("key"), identifier("value"), quote(key), quote(value));
-    exec(command);
+    execOnly(command);
 }
 
-/* Sqlite3Row */
+/* Sqlite3DatabaseWithTransactions */
 
-Sqlite3Row::Sqlite3Row(char** row)
-    : row(row)
+Sqlite3DatabaseWithTransactions::Sqlite3DatabaseWithTransactions(
+    const std::shared_ptr<Config>& config,
+    const std::shared_ptr<Mime>& mime,
+    const std::shared_ptr<ConverterManager>& converterManager,
+    const std::shared_ptr<Timer>& timer)
+    : SqlWithTransactions(config)
+    , Sqlite3Database(config, mime, converterManager, timer)
 {
 }
 
-char* Sqlite3Row::col_c_str(int index) const
+void Sqlite3DatabaseWithTransactions::beginTransaction(std::string_view tName)
 {
-    return row[index];
-}
-
-/* Sqlite3Result */
-
-Sqlite3Result::~Sqlite3Result()
-{
-    if (table) {
-        sqlite3_free_table(table);
-        table = nullptr;
+    if (use_transaction) {
+        log_debug("BEGIN TRANSACTION {} {}", tName, inTransaction);
+        SqlAutoLock lock(sqlMutex);
+        log_debug("BEGIN TRANSACTION LOCK {} {}", tName, inTransaction);
+        StdThreadRunner::waitFor(
+            fmt::format("SqliteDatabase.begin {}", tName), [this] { return !inTransaction; }, 100);
+        inTransaction = true;
+        _exec("BEGIN TRANSACTION");
     }
 }
-std::unique_ptr<SQLRow> Sqlite3Result::nextRow()
+
+void Sqlite3DatabaseWithTransactions::rollback(std::string_view tName)
 {
-    if (nrow) {
-        row += ncolumn;
-        cur_row++;
-        if (cur_row <= nrow) {
-            return std::make_unique<Sqlite3Row>(row);
-        }
-        return nullptr;
+    if (use_transaction && inTransaction) {
+        log_debug("ROLLBACK {} {}", tName, inTransaction);
+        _exec("ROLLBACK");
+        inTransaction = false;
+    }
+}
+
+void Sqlite3DatabaseWithTransactions::commit(std::string_view tName)
+{
+    if (use_transaction && inTransaction) {
+        log_debug("COMMIT {} {}", tName, inTransaction);
+        _exec("COMMIT");
+        inTransaction = false;
     }
-    return nullptr;
 }
