@@ -29,8 +29,8 @@
     $Id$
 */
 
-/// \file transcode_ext_handler.h
-/// \brief Definition of the TranscodeRequest class.
+/// @file transcoding/transcode_ext_handler.h
+/// @brief Definition of the TranscodeRequest class.
 #ifndef __TRANSCODE_EXTERNAL_HANDLER_H__
 #define __TRANSCODE_EXTERNAL_HANDLER_H__
 
@@ -53,7 +53,7 @@ private:
     fs::path makeFifo();
     static void checkTranscoder(const std::shared_ptr<TranscodingProfile>& profile);
 #ifdef HAVE_CURL
-    fs::path openCurlFifo(const fs::path& location, std::vector<std::unique_ptr<ProcListItem>>& procList);
+    fs::path openCurlFifo(const fs::path& location, std::vector<ProcListItem>& procList);
 #endif
 };
 
