@@ -29,7 +29,7 @@
     $Id$
 */
 
-/// \file logger.h
+/// @file util/logger.h
 
 #ifndef __LOGGER_H__
 #define __LOGGER_H__
@@ -86,6 +86,8 @@ enum class GrbLogFacility : unsigned int {
     curl,
     util,
     inotify,
+    thumbnailer,
+    postgres,
     verbose,
 
     log_MAX,
