@@ -29,8 +29,8 @@
     $Id$
 */
 
-/// \file web/pages.h
-/// \brief Defines the various PageRequest sub classes which process requests coming from WebUi.
+/// @file web/pages.h
+/// @brief Defines the various PageRequest sub classes which process requests coming from WebUi.
 #ifndef __WEB_PAGES_H__
 #define __WEB_PAGES_H__
 
@@ -56,22 +56,22 @@ enum class ConfigVal;
 
 namespace Web {
 
-/// \brief Authentication handler (used over AJAX)
+/// @brief Authentication handler (used over AJAX)
 class Auth : public PageRequest {
 protected:
     std::chrono::seconds timeout = std::chrono::seconds::zero();
     bool accountsEnabled { false };
 
     bool processPageAction(Json::Value& element, const std::string& action) override;
-    /// \brief get server configuration parts for UI
+    /// @brief get server configuration parts for UI
     bool getConfig(Json::Value& element);
-    /// \brief get current session id
+    /// @brief get current session id
     bool getSid(Json::Value& element);
-    /// \brief get token for login
+    /// @brief get token for login
     bool getToken(Json::Value& element);
-    /// \brief handle login action
+    /// @brief handle login action
     bool login();
-    /// \brief handle logout action
+    /// @brief handle logout action
     bool logout();
 
 public:
@@ -84,7 +84,7 @@ public:
     std::string_view getPage() const override { return PAGE; }
 };
 
-/// \brief Call from WebUi to create container tree in database view
+/// @brief Call from WebUi to create container tree in database view
 class Containers : public PageRequest {
     using PageRequest::PageRequest;
 
@@ -96,7 +96,7 @@ protected:
     bool processPageAction(Json::Value& element, const std::string& action) override;
 };
 
-/// \brief Call from WebUi to create directory tree in filesystem view
+/// @brief Call from WebUi to create directory tree in filesystem view
 class Directories : public PageRequest {
 public:
     explicit Directories(const std::shared_ptr<Content>& content,
@@ -110,9 +110,9 @@ public:
 
 protected:
     using DirInfo = std::pair<fs::path, bool>;
-    /// \brief get all files in path
+    /// @brief get all files in path
     std::map<std::string, DirInfo> listFiles(const fs::path& path);
-    /// \brief generate xml output for files
+    /// @brief generate xml output for files
     void outputFiles(Json::Value& containers, const std::map<std::string, DirInfo>& filesMap);
 
     bool processPageAction(Json::Value& element, const std::string& action) override;
@@ -120,7 +120,7 @@ protected:
     std::shared_ptr<ConverterManager> converterManager;
 };
 
-/// \brief Call from WebUi to list files in filesystem view
+/// @brief Call from WebUi to list files in filesystem view
 class Files : public PageRequest {
 public:
     explicit Files(const std::shared_ptr<Content>& content,
@@ -138,7 +138,7 @@ protected:
     std::shared_ptr<ConverterManager> converterManager;
 };
 
-/// \brief Call from WebUi to list items in database view
+/// @brief Call from WebUi to list items in database view
 class Items : public PageRequest {
     using PageRequest::PageRequest;
 
@@ -148,14 +148,14 @@ public:
 
 protected:
     bool processPageAction(Json::Value& element, const std::string& action) override;
-    /// \brief run browse according to page request parameters
+    /// @brief run browse according to page request parameters
     std::vector<std::shared_ptr<CdsObject>> doBrowse(
         const std::shared_ptr<CdsObject>& container,
         int start,
         int count,
         Json::Value& items,
         std::string& trackFmt);
-    /// \brief run search according to page request parameters
+    /// @brief run search according to page request parameters
     std::vector<std::shared_ptr<CdsObject>> doSearch(
         const std::shared_ptr<CdsObject>& container,
         int start,
@@ -164,7 +164,7 @@ protected:
         std::string& trackFmt);
 };
 
-/// \brief Call from WebUi to add item from filesystem view
+/// @brief Call from WebUi to add item from filesystem view
 class Add : public PageRequest {
     using PageRequest::PageRequest;
 
@@ -176,7 +176,7 @@ protected:
     bool processPageAction(Json::Value& element, const std::string& action) override;
 };
 
-/// \brief Call from WebUi to Remove item in database view
+/// @brief Call from WebUi to Remove item in database view
 class Remove : public PageRequest {
     using PageRequest::PageRequest;
 
@@ -188,7 +188,7 @@ protected:
     bool processPageAction(Json::Value& element, const std::string& action) override;
 };
 
-/// \brief Call from WebUi to Edit Item in database view
+/// @brief Call from WebUi to Edit Item in database view
 class EditLoad : public PageRequest {
     using PageRequest::PageRequest;
 
@@ -198,36 +198,41 @@ public:
 
 protected:
     bool processPageAction(Json::Value& element, const std::string& action) override;
-    /// \brief write object core info
+    /// @brief write object core info
     void writeCoreInfo(
         const std::shared_ptr<CdsObject>& obj,
         Json::Value& item,
         int objectID);
-    /// \brief write cdsitem info
+    /// @brief write cdsitem info
     void writeItemInfo(
         const std::shared_ptr<CdsItem>& objItem,
         int objectID,
         Json::Value& item,
         Json::Value& metaData);
-    /// \brief write cdscontainer info
+    /// @brief write cdscontainer info
     void writeContainerInfo(
         const std::shared_ptr<CdsObject>& obj,
         Json::Value& item);
-    /// \brief write resource info
+    /// @brief write resource info
     void writeResourceInfo(
         const std::shared_ptr<CdsObject>& obj,
         Json::Value& item);
-    /// \brief write metadata
+    /// @brief write metadata
     void writeMetadata(
         const std::shared_ptr<CdsObject>& obj,
         Json::Value& metadataArray);
-    /// \brief write auxdata
+    /// @brief write auxdata
     void writeAuxData(
         const std::shared_ptr<CdsObject>& obj,
         Json::Value& auxdataArray);
+
+    /// @brief info on referenced objects
+    void writeReferenceInfo(
+        const std::shared_ptr<CdsObject>& obj,
+        Json::Value& item);
 };
 
-/// \brief Call from WebUi to Save Item properties in database view
+/// @brief Call from WebUi to Save Item properties in database view
 class EditSave : public PageRequest {
     using PageRequest::PageRequest;
 
@@ -239,7 +244,7 @@ protected:
     bool processPageAction(Json::Value& element, const std::string& action) override;
 };
 
-/// \brief Call from WebUi to Add Object in database view
+/// @brief Call from WebUi to Add Object in database view
 class AddObject : public PageRequest {
     using PageRequest::PageRequest;
 
@@ -249,29 +254,29 @@ public:
 
 protected:
     bool processPageAction(Json::Value& element, const std::string& action) override;
-    /// \brief handle page request to add a container
+    /// @brief handle page request to add a container
     void addContainer(
         int parentID,
         const std::string& title,
         const std::string& upnp_class);
-    /// \brief handle page request to add a file item
+    /// @brief handle page request to add a file item
     std::shared_ptr<CdsItem> addItem(
         int parentID,
         const std::string& title,
         const std::string& upnp_class,
         const fs::path& location);
-    /// \brief handle page request to add an external url as item
+    /// @brief handle page request to add an external url as item
     std::shared_ptr<CdsItemExternalURL> addUrl(
         int parentID,
         const std::string& title,
         const std::string& upnp_class,
         bool addProtocol,
         const fs::path& location);
-    /// \brief check if file is hidden according to settings
+    /// @brief check if file is hidden according to settings
     bool isHiddenFile(const std::shared_ptr<CdsObject>& cdsObj);
 };
 
-/// \brief Call from WebUi to add or remove autoscan
+/// @brief Call from WebUi to add or remove autoscan
 class Autoscan : public PageRequest {
     using PageRequest::PageRequest;
 
@@ -281,25 +286,25 @@ public:
 
 protected:
     bool processPageAction(Json::Value& element, const std::string& action) override;
-    /// \brief Convert autoscan dir to xml for web response
+    /// @brief Convert autoscan dir to xml for web response
     static void autoscan2XML(
         const std::shared_ptr<AutoscanDirectory>& adir,
         Json::Value& element);
-    /// \brief get details for autoscan editor
+    /// @brief get details for autoscan editor
     void editLoad(
         bool fromFs,
         Json::Value& element,
         const std::string& objID,
         const fs::path& path);
-    /// \brief list all autoscan directories
+    /// @brief list all autoscan directories
     void list(Json::Value& element);
-    /// \brief Save new or changed autoscan
+    /// @brief Save new or changed autoscan
     void editSave(bool fromFs, const fs::path& path);
-    /// \brief Run full scan immediately
+    /// @brief Run full scan immediately
     void runScan(bool fromFs, const fs::path& path);
 };
 
-/// \brief Call from WebUi to do nothing :)
+/// @brief Call from WebUi to do nothing :)
 class VoidType : public PageRequest {
     using PageRequest::PageRequest;
 
@@ -311,7 +316,7 @@ public:
     bool processPageAction(Json::Value& element, const std::string& action) override;
 };
 
-/// \brief task list and task cancel
+/// @brief task list and task cancel
 class Tasks : public PageRequest {
     using PageRequest::PageRequest;
 
@@ -323,7 +328,7 @@ protected:
     bool processPageAction(Json::Value& element, const std::string& action) override;
 };
 
-/// \brief UI action button
+/// @brief UI action button
 class Action : public PageRequest {
     using PageRequest::PageRequest;
 
@@ -335,7 +340,7 @@ protected:
     bool processPageAction(Json::Value& element, const std::string& action) override;
 };
 
-/// \brief Browse clients list
+/// @brief Browse clients list
 class Clients : public PageRequest {
     using PageRequest::PageRequest;
 
@@ -347,7 +352,7 @@ protected:
     bool processPageAction(Json::Value& element, const std::string& action) override;
 };
 
-/// \brief Call from WebUi to load configuration
+/// @brief Call from WebUi to load configuration
 class ConfigLoad : public PageRequest {
 protected:
     std::vector<ConfigValue> dbEntries;
@@ -355,14 +360,14 @@ protected:
     std::shared_ptr<ConfigDefinition> definition;
 
     bool processPageAction(Json::Value& element, const std::string& action) override;
-    /// \brief create config value entry
+    /// @brief create config value entry
     void createItem(
         Json::Value& item,
         const std::string& name,
         ConfigVal id,
         ConfigVal aid,
         const std::shared_ptr<ConfigSetup>& cs = nullptr);
-    /// \brief add value to display in UI
+    /// @brief add value to display in UI
     template <typename T>
     void addValue(
         Json::Value& parent,
@@ -371,7 +376,7 @@ protected:
         ConfigVal aid,
         const T& value,
         const std::shared_ptr<ConfigSetup>& cs = nullptr);
-    /// \brief add value to allow creation of entry in blank config
+    /// @brief add value to allow creation of entry in blank config
     void addNewValue(
         Json::Value& parent,
         const std::string& name,
@@ -379,38 +384,38 @@ protected:
         ConfigVal aid,
         const std::shared_ptr<ConfigSetup>& cs = nullptr);
 
-    /// \brief write values with counters and statitics
+    /// @brief write values with counters and statitics
     void writeDatabaseStatus(Json::Value& values);
-    /// \brief add shortcut configuration
+    /// @brief add shortcut configuration
     void writeShortcuts(Json::Value& values);
-    /// \brief add boolean, integer and string properties
+    /// @brief add boolean, integer and string properties
     void writeSimpleProperties(Json::Value& values);
-    /// \brief add all dictionary configuration options
+    /// @brief add all dictionary configuration options
     void writeDictionaries(Json::Value& values);
-    /// \brief add all vector configuration options
+    /// @brief add all vector configuration options
     void writeVectors(Json::Value& values);
-    /// \brief add all array configuration options
+    /// @brief add all array configuration options
     void writeArrays(Json::Value& values);
-    /// \brief add client configuration
+    /// @brief add client configuration
     void writeClientConfig(Json::Value& values);
-    /// \brief add directory tweaks
+    /// @brief add directory tweaks
     void writeImportTweaks(Json::Value& values);
-    /// \brief add dynamic folder configuration
+    /// @brief add dynamic folder configuration
     void writeDynamicContent(Json::Value& values);
-    /// \brief add box layout
+    /// @brief add box layout
     void writeBoxLayout(Json::Value& values);
-    /// \brief add transcoding
+    /// @brief add transcoding
     void writeTranscoding(Json::Value& values);
-    /// \brief add autoscan details
+    /// @brief add autoscan details
     void writeAutoscan(Json::Value& values);
-    /// \brief overwrite values from xml with database values
+    /// @brief overwrite values from xml with database values
     void updateEntriesFromDatabase(Json::Value& element, Json::Value& values);
 
-    /// \brief write value depending on type
+    /// @brief write value depending on type
     template <typename T>
     static void setValue(Json::Value& item, const T& value);
 
-    /// \brief create meta information on config value
+    /// @brief create meta information on config value
     static void addTypeMeta(Json::Value& meta, const std::shared_ptr<ConfigSetup>& cs);
 
 public:
@@ -423,7 +428,7 @@ public:
     std::string_view getPage() const override { return PAGE; }
 };
 
-/// \brief Call from WebUi to save configuration
+/// @brief Call from WebUi to save configuration
 class ConfigSave : public PageRequest {
 protected:
     std::shared_ptr<Context> context;
