@@ -2,7 +2,7 @@
 
     MediaTomb - http://www.mediatomb.cc/
 
-    web_update.cc - this file is part of MediaTomb.
+    web/web_update.cc - this file is part of MediaTomb.
 
     Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                        Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
@@ -11,7 +11,7 @@
                             Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                             Leonhard Wimmer <leo@mediatomb.cc>
 
-    Copyright (C) 2016-2024 Gerbera Contributors
+    Copyright (C) 2016-2025 Gerbera Contributors
 
     MediaTomb is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2
@@ -29,16 +29,16 @@
     $Id$
 */
 
-/// \file web_update.cc
-#define LOG_FAC log_facility_t::web
+/// @file web/web_update.cc
+#define GRB_LOG_FAC GrbLogFacility::web
 
 #include "pages.h" // API
 
-#include "content/content_manager.h"
-#include "database/database.h"
-#include "util/xml_to_json.h"
+#include "content/content.h"
 
-void Web::VoidType::process()
+const std::string_view Web::VoidType::PAGE = "void";
+
+bool Web::VoidType::processPageAction(Json::Value& element, const std::string& action)
 {
-    checkRequest();
+    return true;
 }
